package org.colinvella.fancyfish.block;

import org.colinvella.fancyfish.tileentity.ModInventoryTileEntity;
import org.colinvella.fancyfish.tileentity.ModTileEntity;

import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;

// equivalent to BlockContainer class but extends ModBlock to the same effect
public abstract class ModContainerBlock extends ModBlock implements ITileEntityProvider
{
	public ModContainerBlock(String id, Material material, CreativeTabs creativeTab)
	{
		super(id, material, creativeTab);
		this.isBlockContainer = true;
	}

	@Override
	public abstract TileEntity createNewTileEntity(World worldIn, int meta); 

	// remove associated tile entity if block destroyed
    @Override
    public void breakBlock(World world, BlockPos pos, IBlockState blockState)
    {
    	// get tile entity
        ModTileEntity modTileEntity = (ModTileEntity) world.getTileEntity(pos);
        
        // if it is an inventory, drop it's contents
        if (modTileEntity instanceof ModInventoryTileEntity)
        {
        	InventoryHelper.dropInventoryItems(world, pos, (ModInventoryTileEntity)modTileEntity);
        }
        
        // call parent for further event processing
        super.breakBlock(world, pos, blockState);
        
        // remove tile entityS
        world.removeTileEntity(pos);
    }
    
    @Override
    public boolean onBlockEventReceived(World worldIn, BlockPos pos, IBlockState state, int eventID, int eventParam)
    {
        super.onBlockEventReceived(worldIn, pos, state, eventID, eventParam);
        TileEntity tileentity = worldIn.getTileEntity(pos);
        return tileentity == null ? false : tileentity.receiveClientEvent(eventID, eventParam);
    }
    
    @Override
    public void onBlockPlacedBy(World worldIn, BlockPos pos, IBlockState state, EntityLivingBase placer, ItemStack stack)
    {
    	if (!stack.hasDisplayName())
    		return;

    	TileEntity tileEntity = worldIn.getTileEntity(pos);
    	if (tileEntity == null)
    		return;
    	
    	if (!(tileEntity instanceof ModInventoryTileEntity))
    		return;
    	
    	ModInventoryTileEntity modInventoryTileEntity = (ModInventoryTileEntity)tileEntity;
    	
    	modInventoryTileEntity.setCustomName(stack.getDisplayName());
    }
}
