package org.colinvella.fancyfish.entity.ai;

import java.util.List;
import java.util.Random;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.entity.fish.FishEntity;
import org.colinvella.fancyfish.logger.ModLogger;

import net.minecraft.entity.EntityAgeable;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityXPOrb;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.stats.StatList;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.world.World;

public class FishEntityAIMate extends EntityAIBase
{
	public FishEntityAIMate(FishEntity fishEntity)
	{
		if (this.logger == null)
			this.logger = FancyFishMod.getLogger();
		
		this.fishEntity = fishEntity;
		this.world = fishEntity.worldObj;
        this.setMutexBits(1);
	}

	@Override
	public boolean shouldExecute()
	{
        if (!this.fishEntity.isInLove())
        {
            return false;
        }
        else
        {
    		this.targetMate = this.getNearbyMate();
            return this.targetMate != null;
        }
    }
	
	@Override
    public boolean continueExecuting()
    {
        return this.targetMate.isEntityAlive() && this.targetMate.isInLove() && this.spawnBabyDelay < 60;
    }

	@Override
    public void resetTask()
    {
        this.targetMate = null;
        this.spawnBabyDelay = 0;
    }

	@Override
    public void updateTask()
    {
        ++this.spawnBabyDelay;

		if (targetMate == null)
			return;
		
        //update distance between fish and target mate
		double distanceSquared = this.fishEntity.getDistanceSqToEntity(this.targetMate);
		
		double distance = Math.sqrt(distanceSquared);
        if (distance < 0.001)
        	distance = 0.001;

    	// get offset from fish to player
        double targetMotionX = targetMate.posX - fishEntity.posX;
        double targetMotionY = targetMate.posY - fishEntity.posY;
        double targetMotionZ = targetMate.posZ - fishEntity.posZ;
         
        // normalise offset to unit vector and multiply by fish's speed
        double factor = fishEntity.getSwimmingSpeed() / distance;
        targetMotionX *= factor;
        targetMotionY *= factor;
        targetMotionZ *= factor;
        
        fishEntity.setTargetMotion(targetMotionX, targetMotionY, targetMotionZ);
		
        if (this.spawnBabyDelay >= 60 && distance < 3.0)
        {
            this.spawnBaby();
        }
    }
    
    private FishEntity getNearbyMate()
    {
        double scanExtent = 8.0;
        AxisAlignedBB scanVolume = this.fishEntity.getEntityBoundingBox().expand(scanExtent, scanExtent, scanExtent);
        List list = this.world.getEntitiesWithinAABB(this.fishEntity.getClass(),scanVolume);
        
        double minSquaredDistance = Double.MAX_VALUE;
        FishEntity nearbyMate = null;
        for (Object item: list)
        {
        	FishEntity potentialMate = (FishEntity)item;
            if (this.fishEntity.canMateWith(potentialMate) && this.fishEntity.getDistanceSqToEntity(potentialMate) < minSquaredDistance)
            {
                nearbyMate = potentialMate;
                minSquaredDistance = this.fishEntity.getDistanceSqToEntity(potentialMate);
            }
        }

        return nearbyMate;
    }
	
    private void spawnBaby()
    {
    	logger.debug("Spawned baby fish of type: " + this.fishEntity.getSpecies());

        EntityAgeable babyFish = this.fishEntity.createChild(this.targetMate);

        if (babyFish != null)
        {
        	// get player that interacted with fish using breeding item
            EntityPlayer entityplayer = this.fishEntity.func_146083_cb();

            if (entityplayer == null && this.targetMate.func_146083_cb() != null)
            {
                entityplayer = this.targetMate.func_146083_cb();
            }

            if (entityplayer != null)
            {
                entityplayer.triggerAchievement(StatList.animalsBredStat);

                // TODO: fish breeding ach here
            }

            double posX = this.fishEntity.posX;
            double posY = this.fishEntity.posY;
            double posZ = this.fishEntity.posZ;

            this.fishEntity.setGrowingAge(6000);
            this.targetMate.setGrowingAge(6000);
            this.fishEntity.resetInLove();
            this.targetMate.resetInLove();
            babyFish.setGrowingAge(-24000);
            babyFish.setLocationAndAngles(posX, posY, posZ, 0.0F, 0.0F);
            this.world.spawnEntityInWorld(babyFish);
            Random random = this.fishEntity.getRNG();
            
            // spawn heart particles
            for (int i = 0; i < 7; ++i)
            {
                double xOffset = random.nextGaussian() * 0.02;
                double yOffset = random.nextGaussian() * 0.02;
                double zOffset = random.nextGaussian() * 0.02;
                this.world.spawnParticle(EnumParticleTypes.HEART, 
            		posX + this.fishEntity.width * (random.nextDouble() * 2.0 - 1.0),
            		posY + 0.5 + random.nextDouble() * this.fishEntity.height,
            		posZ + this.fishEntity.width * (random.nextDouble() * 2.0 - 1.0),
            		xOffset, yOffset, zOffset, new int[0]);
            }

            if (this.world.getGameRules().getGameRuleBooleanValue("doMobLoot"))
            {
                this.world.spawnEntityInWorld(new EntityXPOrb(this.world, posX, posY, posZ, random.nextInt(7) + 1));
            }
        }
    }
    
    private static ModLogger logger;
    
    private FishEntity fishEntity;
    private World world;
    private FishEntity targetMate;    
    private int spawnBabyDelay;
}
