package org.colinvella.fancyfish.item;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.entity.fish.BlennyEntity;
import org.colinvella.fancyfish.entity.fish.ClownFishEntity;
import org.colinvella.fancyfish.entity.fish.DamselFishEntity;
import org.colinvella.fancyfish.entity.fish.MoonFishEntity;
import org.colinvella.fancyfish.entity.fish.MoorishIdolEntity;
import org.colinvella.fancyfish.entity.fish.PufferFishEntity;
import org.colinvella.fancyfish.entity.fish.RegalTangEntity;
import org.colinvella.fancyfish.entity.fish.RoyalGrammaEntity;
import org.colinvella.fancyfish.entity.fish.YellowTangEntity;
import org.colinvella.fancyfish.item.combat.FishScaleArmorItem;
import org.colinvella.fancyfish.item.combat.FishScaleSwordItem;
import org.colinvella.fancyfish.item.combat.ModItemArmor;
import org.colinvella.fancyfish.item.combat.ModItemSword;
import org.colinvella.fancyfish.item.tool.FishScaleAxeItem;
import org.colinvella.fancyfish.item.tool.FishScaleHoeItem;
import org.colinvella.fancyfish.item.tool.FishingNetItem;
import org.colinvella.fancyfish.item.tool.ModItemAxe;
import org.colinvella.fancyfish.item.tool.ModItemHoe;
import org.colinvella.fancyfish.logger.ModLogger;

import net.minecraftforge.fml.common.registry.GameRegistry;

public class ModItemRegistry {
	// general items
    public static final ModItem Netting = new NettingItem();
    public static final ModItem FishBowl = new FishBowlItem();
    public static final ModItem FishFood = new FishFoodItem();
    public static final ModItem Pearl = new PearlItem();
    
    // fish scales
    public static final FishScaleItem RedFishScale = new FishScaleItem(FishScaleItem.Colour.Red);
    public static final FishScaleItem OrangeFishScale = new FishScaleItem(FishScaleItem.Colour.Orange);
    public static final FishScaleItem YellowFishScale = new FishScaleItem(FishScaleItem.Colour.Yellow);
    public static final FishScaleItem GreenFishScale = new FishScaleItem(FishScaleItem.Colour.Green);
    public static final FishScaleItem BlueFishScale = new FishScaleItem(FishScaleItem.Colour.Blue);
    public static final FishScaleItem IndigoFishScale = new FishScaleItem(FishScaleItem.Colour.Indigo);
    public static final FishScaleItem VioletFishScale = new FishScaleItem(FishScaleItem.Colour.Violet);
    public static final FishScaleItem BlackFishScale = new FishScaleItem(FishScaleItem.Colour.Black);
    public static final FishScaleItem WhiteFishScale = new FishScaleItem(FishScaleItem.Colour.White);
    public static final FishScaleItem SilverFishScale = new FishScaleItem(FishScaleItem.Colour.Silver);
    
    // captured fish
    public static final ModItem CapturedClownFish = new CapturedFishItem("CapturedClownFish", ClownFishEntity.class);
    public static final ModItem CapturedRegalTang = new CapturedFishItem("CapturedRegalTang", RegalTangEntity.class);
    public static final ModItem CapturedMoorishIdol = new CapturedFishItem("CapturedMoorishIdol", MoorishIdolEntity.class);
    public static final ModItem CapturedRoyalGramma = new CapturedFishItem("CapturedRoyalGramma", RoyalGrammaEntity.class);
    public static final ModItem CapturedPufferFish = new CapturedFishItem("CapturedPufferFish", PufferFishEntity.class);
    public static final ModItem CapturedYellowTang = new CapturedFishItem("CapturedYellowTang", YellowTangEntity.class);
    public static final ModItem CapturedDamselFish = new CapturedFishItem("CapturedDamselFish", DamselFishEntity.class);
    public static final ModItem CapturedBlenny = new CapturedFishItem("CapturedBlenny", BlennyEntity.class);
    public static final ModItem CapturedMoonFish = new CapturedFishItem("CapturedMoonFish", MoonFishEntity.class);
    
    // live fish
    public static final ModItem LiveClownFish = new LiveFishItem("LiveClownFish", ClownFishEntity.class);
    public static final ModItem LiveRegalTang = new LiveFishItem("LiveRegalTang", RegalTangEntity.class);
    public static final ModItem LiveMoorishIdol = new LiveFishItem("LiveMoorishIdol", MoorishIdolEntity.class);
    public static final ModItem LiveRoyalGramma = new LiveFishItem("LiveRoyalGramma", RoyalGrammaEntity.class);
    public static final ModItem LivePufferFish = new LiveFishItem("LivePufferFish", PufferFishEntity.class);
    public static final ModItem LiveYellowTang = new LiveFishItem("LiveYellowTang", YellowTangEntity.class);
    public static final ModItem LiveDamselFish = new LiveFishItem("LiveDamselFish", DamselFishEntity.class);
    public static final ModItem LiveBlenny = new LiveFishItem("LiveBlenny", BlennyEntity.class);
    public static final ModItem LiveMoonFish = new LiveFishItem("LiveMoonFish", MoonFishEntity.class);
    
    // tools
    public static final ModItem FishingNet = new FishingNetItem();
    public static final ModItemAxe FishScaleAxe = new FishScaleAxeItem();
    public static final ModItemHoe FishScaleHoe = new FishScaleHoeItem();
    
    // armour
    public static final ModItemArmor FishScaleHelmet = new FishScaleArmorItem("FishScaleHelmet", 1, 0);
    public static final ModItemArmor FishScaleChestPlate = new FishScaleArmorItem("FishScaleChestPlate", 1, 1);
    public static final ModItemArmor FishScaleLeggings = new FishScaleArmorItem("FishScaleLeggings", 2, 2);
    public static final ModItemArmor FishScaleBoots = new FishScaleArmorItem("FishScaleBoots", 1, 3);
    
    // sword
    public static final ModItemSword FishScaleSword = new FishScaleSwordItem();

    public static LiveFishItem getLiveFromCaptured(CapturedFishItem capturedFishItem)
    {
        String liveFishFieldName = capturedFishItem.getId().replace("Captured", "Live");
        try
        {
            Field field = ModItemRegistry.class.getField(liveFishFieldName);
            return (LiveFishItem)field.get(null);
        }
        catch (Exception eception)
        {
            return null;
        }
    }

    public static CapturedFishItem getCapturedFromLive(LiveFishItem liveFishItem)
    {
        String capturedFishFieldName = liveFishItem.getId().replace("Live", "Captured");
        try
        {
            Field field = ModItemRegistry.class.getField(capturedFishFieldName);
            return (CapturedFishItem)field.get(null);
        }
        catch (Exception eception)
        {
            return null;
        }
    }

    public static List<ModItem> getModItems()
    {
        return Collections.unmodifiableList(modItems);
    }

    public static List<ModItemAxe> getModItemAxes()
    {
        return Collections.unmodifiableList(modItemAxes);
    }

    public static List<ModItemHoe> getModItemHoes()
    {
        return Collections.unmodifiableList(modItemHoes);
    }

    public static List<ModItemArmor> getModItemArmors()
    {
        return Collections.unmodifiableList(modItemArmors);
    }

    public static List<ModItemSword> getModItemSwords()
    {
        return Collections.unmodifiableList(modItemSwords);
    }

    public static void registerItems()
    {
        logger = FancyFishMod.getLogger();
        logger.info("Registering items...");

        // various
        ModItemRegistry.registerItem(Netting);
        ModItemRegistry.registerItem(FishBowl);
        ModItemRegistry.registerItem(FishFood);
        ModItemRegistry.registerItem(Pearl);

        // tools
        ModItemRegistry.registerItem(FishingNet);
        ModItemRegistry.registerAxeItem(FishScaleAxe);
        ModItemRegistry.registerHoeItem(FishScaleHoe);
        
        // fish scales
        ModItemRegistry.registerItem(RedFishScale);
        ModItemRegistry.registerItem(OrangeFishScale);
        ModItemRegistry.registerItem(YellowFishScale);
        ModItemRegistry.registerItem(GreenFishScale);
        ModItemRegistry.registerItem(BlueFishScale);
        ModItemRegistry.registerItem(IndigoFishScale);
        ModItemRegistry.registerItem(VioletFishScale);
        ModItemRegistry.registerItem(BlackFishScale);
        ModItemRegistry.registerItem(WhiteFishScale);
        ModItemRegistry.registerItem(SilverFishScale);
        
        // capture fish items
        ModItemRegistry.registerItem(CapturedClownFish);
        ModItemRegistry.registerItem(CapturedRegalTang);
        ModItemRegistry.registerItem(CapturedMoorishIdol);
        ModItemRegistry.registerItem(CapturedRoyalGramma);
        ModItemRegistry.registerItem(CapturedPufferFish);
        ModItemRegistry.registerItem(CapturedYellowTang);
        ModItemRegistry.registerItem(CapturedDamselFish);
        ModItemRegistry.registerItem(CapturedBlenny);
        ModItemRegistry.registerItem(CapturedMoonFish);
        
        // live fish items
        ModItemRegistry.registerItem(LiveClownFish);
        ModItemRegistry.registerItem(LiveRegalTang);
        ModItemRegistry.registerItem(LiveMoorishIdol);
        ModItemRegistry.registerItem(LiveRoyalGramma);
        ModItemRegistry.registerItem(LivePufferFish);
        ModItemRegistry.registerItem(LiveYellowTang);
        ModItemRegistry.registerItem(LiveDamselFish);
        ModItemRegistry.registerItem(LiveBlenny);
        ModItemRegistry.registerItem(LiveMoonFish);
        
        // armour
        ModItemRegistry.registerArmorItem(FishScaleHelmet);
        ModItemRegistry.registerArmorItem(FishScaleChestPlate);
        ModItemRegistry.registerArmorItem(FishScaleLeggings);
        ModItemRegistry.registerArmorItem(FishScaleBoots);
        
        ModItemRegistry.registerSwordItem(FishScaleSword);
    }

    private static void registerItem(ModItem modItem)
    {
        String id = modItem.getId();
        logger.info("Registering item " + id + "...");
        GameRegistry.registerItem(modItem, id);
        modItems.add(modItem);
    }

    private static void registerAxeItem(ModItemAxe modItemAxe)
    {
        String id = modItemAxe.getId();
        logger.info("Registering axe item " + id + "...");
        GameRegistry.registerItem(modItemAxe, id);
        modItemAxes.add(modItemAxe);
    }

    private static void registerHoeItem(ModItemHoe modItemHoe)
    {
        String id = modItemHoe.getId();
        logger.info("Registering hoe item " + id + "...");
        GameRegistry.registerItem(modItemHoe, id);
        modItemHoes.add(modItemHoe);
    }

    private static void registerArmorItem(ModItemArmor modItemArmor)
    {
        String id = modItemArmor.getId();
        logger.info("Registering armor item " + id + "...");
        GameRegistry.registerItem(modItemArmor, id);
        modItemArmors.add(modItemArmor);
    }

    private static void registerSwordItem(ModItemSword modItemSword)
    {
        String id = modItemSword.getId();
        logger.info("Registering sword item " + id + "...");
        GameRegistry.registerItem(modItemSword, id);
        modItemSwords.add(modItemSword);
    }
    
    private static ModLogger logger;
    
    private static List<ModItem> modItems = new ArrayList<ModItem>();
    private static List<ModItemAxe> modItemAxes = new ArrayList<ModItemAxe>();
    private static List<ModItemHoe> modItemHoes = new ArrayList<ModItemHoe>();
    private static List<ModItemArmor> modItemArmors = new ArrayList<ModItemArmor>();
    private static List<ModItemSword> modItemSwords = new ArrayList<ModItemSword>();
}

