package org.colinvella.fancyfish.tileentity;

import java.util.ArrayList;
import java.util.List;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.item.FishScaleItem;

import com.sun.javafx.geom.Vec3d;

import net.minecraft.block.Block;
import net.minecraft.block.BlockRailBase;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityMinecart;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.Packet;
import net.minecraft.network.play.server.S35PacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;

public class TeleportTileEntity extends ModInventoryTileEntity
{
	public TeleportTileEntity()
	{
		super();
	}

	// elongated render AABB due to beam
	@Override public AxisAlignedBB getRenderBoundingBox()
	{
		AxisAlignedBB aabb = super.getRenderBoundingBox();
		return aabb.expand(0, 16, 0);
	};
	
	@Override
	public int getInventoryStackLimit()
	{
		return 1;
	}
	
	@Override
	public int getSizeInventory()
	{
		return 1;
	}
	
	// stop hoppers from putting anything other than fish scales
	@Override
	public boolean isItemValidForSlot(int index, ItemStack stack)
	{
		if (stack == null)
			return false;
		
		return stack.getItem() instanceof FishScaleItem;
	}
	
	@Override public void onChunkUnload()
	{
		teleportPositions.remove(this.pos);
		logger.debug("Removed teleport at " + this.pos);
	};
	
	@Override
	public void update()
	{
		List entitiesOnTeleport = worldObj.getEntitiesWithinAABB(
				Entity.class,
				new AxisAlignedBB(pos.getX(), pos.getY(), pos.getZ(), pos.getX() + 1, pos.getY() + 3, pos.getZ() + 1));

		float deltaTime = 0.05f; // assume 20 ticks/s

		// charge only if clear and has fish scale
		if (entitiesOnTeleport.size() == 0 && this.beam == 0)
		{
			float oldCharge = this.charge;
			this.charge = Math.min(1.0f, this.charge + deltaTime);
			
			if (charge != oldCharge)
			{
				this.worldObj.markBlockForUpdate(pos);
			}
		}

		// discharge if no fish scale in teleport
		if (this.getStackInSlot(0) == null)
			this.charge = 0;

		// if beam value not zero, converge towards zero
		// beam sign is used to distinguish between incoming and outgoing effect
		if (beam > 0)
			beam = Math.max(0, beam - deltaTime);
		else if (beam < 0)
			beam = Math.min(1, beam + deltaTime);

		if (this.worldObj.isRemote)
		{
			if (charge < 1.0)
			{
				//this.worldObj.spawnParticle(EnumParticleTypes.CLOUD, pos.getX() + 0.5, pos.getY() + 1, pos.getZ() + 0.5, 0, 0.1, 0);
			}
		}
		else
		{
			// server side			
			if (teleportPositions.indexOf(this.pos) < 0)
			{
				teleportPositions.add(this.pos);
				logger.debug("Registered teleport at " + this.pos);
			}
		}
		
		super.update();
	}
	
	// sync tile entities between client and server (for TESR animation)
	
	// serialise
    @Override
    public Packet getDescriptionPacket()
    {
        NBTTagCompound syncData = new NBTTagCompound();
        this.writeToNBT(syncData);
        return new S35PacketUpdateTileEntity(this.pos, this.getBlockMetadata(), syncData);
    }
    
    // deserialise
    @Override
    public void onDataPacket(NetworkManager net, S35PacketUpdateTileEntity pkt)
    {
	    this.readFromNBT(pkt.getNbtCompound());
    }		
    
    // override NBT serialisation to include teleport properties
    @Override
    public void readFromNBT(NBTTagCompound nbt)
    {
    	super.readFromNBT(nbt);
    	this.charge = nbt.getFloat("Charge");
    	this.beam = nbt.getFloat("Beam");
    };
    
    @Override
    public void writeToNBT(NBTTagCompound nbt)
    {
    	super.writeToNBT(nbt);
    	nbt.setFloat("Charge", this.charge);
    	nbt.setFloat("Beam", this.beam);
    };
    
    public float getCharge()
    {
    	return this.charge;
    }

    public float getBeam()
    {
    	return this.beam;
    }

    public void attemptEntityTeleport(Entity entity)
    {
    	// do nothing if not fully charged
    	if (charge < 1.0f)
    		return;

    	// get destination teleport
    	TeleportTileEntity destinationTeleport = getDestinationTeleport();
    	
    	// do nothing if no destination available
    	if (destinationTeleport == null)
    	{
    		this.charge = 0.9f;
    		entity.addChatMessage(new ChatComponentTranslation("container.TeleportTileEntity.message.noDestination"));
    		
    		return;
    	}
    	
    	// ensure destination fully charged
    	if (destinationTeleport.charge < 1.0)
    		return;
    	
    	// discharge both teleports
    	this.charge = destinationTeleport.charge = 0f;
    	this.beam = 0.5f;
    	destinationTeleport.beam = -0.5f;
    	
    	EnumFacing facing = entity.getHorizontalFacing();
    	
    	// transfer entity to destination position
		BlockPos destinationPos = destinationTeleport.getPos().up();

		double destPosX = destinationPos.getX() + 0.5;
		double destPosY = destinationPos.getY();
		double destPosZ = destinationPos.getZ() + 0.5;
		
		// if entity is a minecart, transfer it to an adjacent rail at the destination
		if (entity instanceof EntityMinecart)
		{
			if (worldObj.getBlockState(destinationPos.west()).getBlock() instanceof BlockRailBase)
			{
				//destinationPos = destinationPos.west();
				destPosX += 2.75;
				entity.setVelocity(0.05, 0, 0);
			}
			else if (worldObj.getBlockState(destinationPos.east()).getBlock() instanceof BlockRailBase)
			{
				//destinationPos = destinationPos.east();
				destPosX -= 2.75;
				entity.setVelocity(-0.05, 0, 0);
			}
			else if (worldObj.getBlockState(destinationPos.north()).getBlock() instanceof BlockRailBase)
			{
				//destinationPos = destinationPos.north();
				destPosZ -= 0.5;
				entity.setVelocity(0, 0, -0.05);
			}
			else if (worldObj.getBlockState(destinationPos.south()).getBlock() instanceof BlockRailBase)
			{
				//destinationPos = destinationPos.south();
				destPosZ += 0.5;
				entity.setVelocity(0, 0, 0.05);
			}
		}

		entity.setPositionAndUpdate(destPosX, destPosY, destPosZ);
		
    	// play transfer sound at source and destination
    	double soundX = pos.getX() + 0.5; 
    	double soundY = pos.getY() + 0.5; 
    	double soundZ = pos.getZ() + 0.5; 
    	worldObj.playSoundEffect(soundX, soundY, soundZ, TELEPORT_TRANSFER, 1F, 1F);
    	
    	soundX = destinationPos.getX() + 0.5;
    	soundY = destinationPos.getY() + 0.5;
    	soundZ = destinationPos.getZ() + 0.5;
    	worldObj.playSoundEffect(soundX, soundY, soundZ, TELEPORT_TRANSFER, 1F, 0.8F);

		// sync to client
		this.worldObj.markBlockForUpdate(pos);
		this.worldObj.markBlockForUpdate(destinationPos.down());
		//this.markDirty();
		//destinationTeleport.markDirty();
		
		logger.debug("Teleported entity " + entity + " to " + destinationPos);
    }

    private TeleportTileEntity getDestinationTeleport()
    {
    	// if this teleport has no fish scale, no destination
    	ItemStack sourceItemStack = this.getStackInSlot(0);
    	if (sourceItemStack == null)
    		return null;
    	
    	// get source fish scale
    	FishScaleItem sourceFishScaleItem = (FishScaleItem) sourceItemStack.getItem();
    	
    	// if this teleport is not indexed for some odd reason, no destination
    	int teleportIndex = teleportPositions.indexOf(this.pos);
    	if (teleportIndex < 0)
    		return null;
    	
    	// trivially, if there is only one teleport, no destination
    	int teleportCount = teleportPositions.size();
    	if (teleportCount == 1)
    		return null;
    	
    	// scan for next teleport with matching fish scale
    	int nextIndex = teleportIndex;
    	while(true)
    	{
    		// get next teleport in list
    		nextIndex = (nextIndex + 1) % teleportCount;
    		
    		// if we looped around, no destination
    		if (nextIndex == teleportIndex)
    			return null;

    		// get tile entity at this index
        	TileEntity destinationTileEntity = worldObj.getTileEntity(teleportPositions.get(nextIndex));
        	
        	// if not teleport entity for some reason, remove from list and no destination
        	// (should retry in next update iteration)
        	if (!(destinationTileEntity instanceof TeleportTileEntity))
        	{
        		teleportPositions.remove(nextIndex);
        		return null;
        	}
        	
        	// cast to teleport entity
        	TeleportTileEntity destinationTeleportTileEntity = (TeleportTileEntity) destinationTileEntity;

        	// get item stack and if null, scan next
    		ItemStack destinationItemStack = destinationTeleportTileEntity.getStackInSlot(0);
    		if (destinationItemStack == null)
    			continue;
    		
        	// get destination fish scale
        	FishScaleItem destinationFishScaleItem = (FishScaleItem) destinationItemStack.getItem();
        	
        	// if fish scales do not match, keep scanning
        	if (sourceFishScaleItem != destinationFishScaleItem)
        		continue;
    		
        	// all checks passed, use this teleport
        	return destinationTeleportTileEntity;
    	}
    }

    private float charge = 0;
    private float beam = 0;
    
    private static List<BlockPos> teleportPositions = new ArrayList<BlockPos>();    
    
    private static final String TELEPORT_TRANSFER = FancyFishMod.MOD_ID + ":block.teleport.Transfer";
}
