package org.colinvella.fancyfish.tileentity.renderer;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.item.FishScaleItem;
import org.colinvella.fancyfish.item.FishScaleItem.Colour;
import org.colinvella.fancyfish.item.ModItemRegistry;
import org.colinvella.fancyfish.tileentity.TeleportTileEntity;
import org.colinvella.fancyfish.tileentity.model.TeleportGlowModel;
import org.lwjgl.opengl.GL11;

import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.entity.Entity;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.CLIENT)
public class TeleportTileEntityRenderer extends TileEntitySpecialRenderer
{ 
	@Override
    public void renderTileEntityAt(TileEntity tileEntity, double posX, double posZ, double p_180535_6_, float p_180535_8_, int p_180535_9_)
    {
        this.renderFishtankTileEntity((TeleportTileEntity)tileEntity, posX, posZ, p_180535_6_, p_180535_8_, p_180535_9_);
    }

    private void renderFishtankTileEntity(TeleportTileEntity teleportTileEntity,
    	double x, double y, double z, float partialTicks, int unknownInt)
    {
    	// get fish scale item stored in teleport
    	// render nothing if not set
        ItemStack itemStack = teleportTileEntity.getStackInSlot(0);
        if (itemStack == null)
        	return;
        
        FishScaleItem fishScaleItem = (FishScaleItem) itemStack.getItem();
    	
        GlStateManager.pushMatrix();
        GlStateManager.pushAttrib();

        this.bindTexture(glowTexture);
        
        GlStateManager.translate(x, y + 1, z);
        GlStateManager.depthMask(false);

        // also enable blend directly due to statemanager cache bug
        GlStateManager.enableBlend();
        GL11.glEnable(GL11.GL_BLEND);
        
        // alpha blending for black scale, additive for the rest
        GlStateManager.blendFunc(GL11.GL_SRC_ALPHA,
        	fishScaleItem == ModItemRegistry.BlackFishScale ? GL11.GL_ONE_MINUS_SRC_ALPHA : GL11.GL_ONE);

        float beam = teleportTileEntity.getBeam();
        if (beam == 0)
        {
        	GlStateManager.scale(1, teleportTileEntity.getCharge(), 1);
        }
        else
        {
        	float absBeam = Math.abs(beam);
        	float scale = absBeam * 2f;
        	if (beam < 0)
        		scale = 1f - scale;
        	float scaleOffset = (1f -scale) * 0.5f;
        	GlStateManager.translate(scaleOffset, 0, scaleOffset);
        	GlStateManager.scale(scale, (1 - scale) * 16, scale);
        }
        
        // get beam colour from teleport fish scale
        Colour colour = fishScaleItem.getColour();
        GlStateManager.color(colour.red, colour.green, colour.blue);
        
        float time = 0;
        float limbSwingDistance = 0;
        float unknown = 0;
        float headYRotation = 0;
        float headXRotation = 0.0f;
        float yTranslation = 0.0625F;
        teleportGlowModel.render((Entity)null, time, limbSwingDistance, unknown, headYRotation, headXRotation, yTranslation);

        GlStateManager.popAttrib();
        GlStateManager.popMatrix();
    }
    
    private TeleportGlowModel teleportGlowModel = new TeleportGlowModel();

    private static final ResourceLocation glowTexture = new ResourceLocation(
        FancyFishMod.MOD_ID + ":textures/blocks/TeleportGlow.png");
}